import os
import numpy as np
from tests.data import voyager_h5, voyager_fil
import blimpy as bl
from blimpy.waterfall import cmd_tool

import pytest

OUTDIR = os.path.dirname(voyager_h5) + "/"

def test_info():
    a = bl.Waterfall(voyager_h5)
    print(a)
    a.info()
    a.blank_dc(n_coarse_chan=1)
    a.calibrate_band_pass_N1()
    # Below: orphaned functions (not used anywhere else).
    # That is a little strange...
    a.grab_data()
    a.read_data()
    # It is VERY strange for INTERNAL functions to be orphaned!
    a._get_chunk_dimensions()
    # plenty of missed if suites
    a._get_blob_dimensions((300, 300, 300, 300))
    a._update_header()
    del a

def test_get_freqs():
    wf = bl.Waterfall(voyager_h5)
    freqs = wf.container.populate_freqs()
    sum1 = np.sum(freqs)
    freqs = wf.get_freqs()
    sum2 = np.sum(freqs)
    assert sum1 == sum2
    wf = bl.Waterfall(voyager_fil)
    freqs = wf.container.populate_freqs()
    sum1 = np.sum(freqs)
    freqs = wf.get_freqs()
    sum2 = np.sum(freqs)
    assert sum1 == sum2
    len_f = len(freqs)
    first = wf.header["fch1"]
    last_1 = freqs[-1]
    last_2 = first + (len_f - 1 ) *  wf.header["foff"]
    assert np.isclose(last_1, last_2, rtol=0.0001)

def test_cmdline():
    # Plots
    
    args = [voyager_h5, '-S', '-p', 'w', '-s', OUTDIR + 'test.png']
    cmd_tool(args)

    args = [voyager_h5, '-S', '-p', 's', '-s', OUTDIR + 'test.png']
    cmd_tool(args)

    args = [voyager_h5, '-S', '-p', 'mm', '-s', OUTDIR + 'test.png']
    cmd_tool(args)

    args = [voyager_h5, '-S', '-p', 'k', '-s', OUTDIR + 'test.png']
    cmd_tool(args)

    args = [voyager_h5, '-S', '-p', 't', '-s', OUTDIR + 'test.png']
    cmd_tool(args)

    args = [voyager_h5, '-S', '-p', 'a', '-s', OUTDIR + 'test.png']
    cmd_tool(args)

    args = [voyager_h5, '-S', '-p', 'ank', '-s', OUTDIR + 'test.png']
    cmd_tool(args)
    
    args = [voyager_h5, '-S', '-p', 'ank', '-s', OUTDIR + 'test.png']
    cmd_tool(args)
    
    # Blank DC to .h5

    args = [voyager_h5, '-D', '-H', '-o', OUTDIR + 'test.h5']
    cmd_tool(args)

    # Blank DC to .fil

    args = [voyager_h5, '-D', '-F', '-o', OUTDIR + 'test.fil']
    cmd_tool(args)
    
    # info with foff negative

    args = [voyager_h5, '-i']
    cmd_tool(args)

    if os.path.exists(OUTDIR + 'test.h5'):
        os.remove(OUTDIR + 'test.h5')
        args = [voyager_h5, '-H', '-o', OUTDIR + 'test.h5']
        cmd_tool(args)
        assert os.path.exists(OUTDIR + 'test.h5')
        os.remove(OUTDIR + 'test.h5')

    if os.path.exists(OUTDIR + 'test.fil'):
        os.remove(OUTDIR + 'test.fil')
        args = [voyager_h5, '-F', '-o', OUTDIR + 'test.fil']
        cmd_tool(args)
        assert os.path.exists(OUTDIR + 'test.fil')
        os.remove(OUTDIR + 'test.fil')

def test_cmd_arguments():
    args = [voyager_h5, '-H', '-F', '-o', OUTDIR + 'test.fil']
    with pytest.raises(ValueError):
        cmd_tool(args)

def test_neg_blank_dc():
    wf = bl.Waterfall(voyager_h5)
    wf.blank_dc(0)
    wf.blank_dc(1.1)
    del wf

def test_get_chunk_dimensions():
    wf = bl.Waterfall(voyager_h5)
    
    wf.header['foff'] = 0.99e-5
    assert wf._get_chunk_dimensions() == (1, 1, 1048576)
    wf.header['foff'] = 1.1e-5
    
    wf.header['tsamp'] = 0.99e-3
    assert wf._get_chunk_dimensions() == (2048, 1, 512)
    wf.header['tsamp'] = 1.1e-3
    
    wf.header['foff'] = 0.99e-2
    assert wf._get_chunk_dimensions() == (10, 1, 65536)
    
    wf.header['foff'] = 1e-1
    assert wf._get_chunk_dimensions() == (1, 1, 512)
    
    del wf

def test_neg_info_foff():
    wf = bl.Waterfall(voyager_h5)
    wf.header['foff'] = -1
    wf.info()
    del wf

def test_no_filename():
    header = { "banana": "is a fruit" }
    data = np.arange(1, 11)
    class Puddle(bl.Waterfall):
        def _setup_freqs(self):
            print("Hello from function _setup_freqs")
    puddle = Puddle(header_dict=header, data_array=data)
    puddle = Puddle(header_dict=header)
    del puddle

test_cmdline()





import blimpy as bl
import numpy as np
from pprint import pprint
from tests.data import voyager_fil, voyager_h5
from blimpy.plotting.config import plt


def test_compare_waterfall_fil_to_h5():
    """ Load Voyager dataset and test that both fil and hdf5 readers return same headers and data """

    print("Loading FIL and HDF5 data with Waterfall()..."),
    a = bl.Waterfall(voyager_h5)
    b = bl.Waterfall(voyager_fil)

    print("Reading headers..")
    pprint(a.header)
    print("\nFIL file header:")
    pprint(b.header)
    print("Headers are loading OK")

    print("\nChecking header values match..."),
    for key in b.header.keys():
        assert b.header[key] == a.header[key]

    print("Checking datatype matches..."),
    assert a.data.dtype == b.data.dtype

    print("Checking data matches..."),
    assert np.allclose(a.data, b.data)
    assert a.data.dtype == b.data.dtype


def test_waterfall_fil_to_h5_methods_and_attributes():
    """ Compare attributes and check methods """
    a = bl.Waterfall(voyager_h5)
    b = bl.Waterfall(voyager_fil)

    print("Comparing attributes of classes match where expected")
    assert a.beam_axis == b.beam_axis
    assert a.freq_axis == b.freq_axis
    assert a.time_axis == b.time_axis

    assert a.calc_n_coarse_chan() == b.calc_n_coarse_chan()
    assert a.file_shape == b.file_shape

    assert a.n_channels_in_file == b.n_channels_in_file
    assert a.n_ints_in_file == b.n_ints_in_file
    assert a.selection_shape == b.selection_shape

    print("Checking if basic methods run without raising Exceptions")
    # Check they can be run
    a.container.populate_freqs()
    a.container.populate_timestamps()
    a.info()
    a.blank_dc(1)
    a.calibrate_band_pass_N1()

    b.container.populate_freqs()
    b.container.populate_timestamps()
    b.info()
    b.blank_dc(1)
    b.calibrate_band_pass_N1()

    dir_a = dir(a)
    dir_b = dir(b)

    print("Attr/methods in HDF5 but not in FIL:")
    for item in dir_a:
        if item not in dir_b:
            raise ValueError("HDF5 item is not in FIL:" + str(item))

    print("Attr/methods in FIL but not in HDF5:")
    for item in dir_b:
        if item not in dir_a:
            raise ValueError("FIL item is not in HDF5:" + str(item))


def test_plotting_doesnt_cause_exceptions():
    """ Try running the plotting routines. They should not raise expections even without X windows """
    a = bl.Waterfall(voyager_h5)
    b = bl.Waterfall(voyager_fil)

    a.plot_all()
    plt.clf()
    a.plot_kurtosis()
    plt.clf()
    a.plot_spectrum()
    plt.clf()
    a.plot_spectrum_min_max()
    plt.clf()
    a.plot_waterfall()
    plt.clf()
    a.plot_time_series()
    plt.clf()

    b.plot_all()
    plt.clf()
    b.plot_kurtosis()
    plt.clf()
    b.plot_spectrum()
    plt.clf()
    b.plot_spectrum_min_max()
    plt.clf()
    b.plot_waterfall()
    plt.clf()
    b.plot_time_series()
    plt.clf()




from os.path import dirname
import numpy as np
import hdf5plugin
import h5py
from blimpy.h5diag import cmd_tool
from tests.data import voyager_h5, voyager_fil
import pytest


header = [
    ["fruit", "apple"],
    ["color", "red"],
    ["plant", "tree"]
    ]
DIR = dirname(voyager_fil)
TEST_H5 = DIR + "/test.h5"
TIME_INSTANCES = 8
FREQ_INSTANCES = 16
DATA_BYTESIZE = TIME_INSTANCES * FREQ_INSTANCES * 4


def my_writer(my_class):
    data_out = np.ndarray(shape=(TIME_INSTANCES, 1, FREQ_INSTANCES), dtype=float)
    for ii in range(TIME_INSTANCES):
        for jj in range(FREQ_INSTANCES):
            data_out[ii, 0, jj] = 42.0
    print("data_out shape:", data_out.shape)
    
    with h5py.File(TEST_H5, "w") as h5:
        h5.attrs["CLASS"]   = my_class
        h5.attrs["VERSION"] = "1.0"
    
        bs_compression = hdf5plugin.Bitshuffle(nelems=0, lz4=True)["compression"]
        bs_compression_opts = hdf5plugin.Bitshuffle(nelems=0, lz4=True)["compression_opts"]
    
        dset = h5.create_dataset("data",
                                 data=data_out,
                                 compression=bs_compression,
                                 compression_opts=bs_compression_opts)
    
        dset_mask = h5.create_dataset("mask",
                                      shape=data_out.shape,
                                      compression=bs_compression,
                                      compression_opts=bs_compression_opts,
                                      dtype="uint8")
    
        dset.dims[2].label = b"frequency"
        dset.dims[1].label = b"feed_id"
        dset.dims[0].label = b"time"
    
        dset_mask.dims[2].label = b"frequency"
        dset_mask.dims[1].label = b"feed_id"
        dset_mask.dims[0].label = b"time"
    
        # Copy over header information as attributes
        for key, value in header:
            dset.attrs[key] = value


def execute_command(args):
    print("\ntest_h5diag: args:", args)
    cmd_tool(args)


def test_h5diag():

    args = [voyager_h5]
    execute_command(args)

    with pytest.raises(SystemExit):
        args = [voyager_fil]
        execute_command(args)
    
    my_writer("FRUITY")
    with pytest.raises(SystemExit):
        args = [TEST_H5]
        execute_command(args)
    
    with h5py.File(TEST_H5, "w") as h5:
        h5.attrs["VERSION"] = "42.0"
    with pytest.raises(SystemExit):
        args = [TEST_H5]
        execute_command(args)
    
    with h5py.File(TEST_H5, "w") as h5:
        h5.attrs["CLASS"] = "FILTERBANK"
    with pytest.raises(SystemExit):
        args = [TEST_H5]
        execute_command(args)

    with h5py.File(TEST_H5, "w") as h5:
        h5.attrs["CLASS"] = "FILTERBANK"
        h5.attrs["VERSION"] = "42.0"
    with pytest.raises(SystemExit):
        args = [TEST_H5]
        execute_command(args)

    my_writer("FILTERBANK")
    args = [TEST_H5]
    execute_command(args)


if __name__ == "__main__":
    test_h5diag()



""" srcname 

    Change the HDF5 Filterbank header source_name field value.
"""

import os
import sys
from argparse import ArgumentParser
import h5py
from astropy.coordinates import Angle


def oops(msg):
    print("\n*** srcname: {}\n".format(msg))
    sys.exit(86)


def read_header(h5):
    """
    Read the Filterbank header and return a Python dictionary of key:value pairs

    Parameters
    ----------
    h5 : HDF5 file handle
        This represents an open HDF5 file.

    Returns
    -------
    header : dict
        The Filterbank header.

    """
    header = {} # Initialise as a nil dictionary.

    for key, val in h5["data"].attrs.items():
        if isinstance(val, bytes):
            val = val.decode("ascii")
        if key == "src_raj":
            header[key] = Angle(val, unit="hr")
        elif key == "src_dej":
            header[key] = Angle(val, unit="deg")
        else:
            header[key] = val

    return header


def examine(filepath):
    """
    Diagnose the specified HDF5 file.

    Parameters
    ----------
    filepath : path
        An O/S path to an HDF5 file.

    Returns
    -------
    header : dict
        The Filterbank header.

    """
    h5 = h5py.File(filepath, mode="r")
    if "CLASS" in h5.attrs:
        classstr = h5.attrs["CLASS"]
    else:
        oops("CLASS attribute missing")
    if classstr != "FILTERBANK":
        oops("Expected CLASS attribute to be 'FILTERBANK' but saw '{}'".format(classstr))
    if "VERSION" in h5.attrs:
        versionstr = h5.attrs["VERSION"]
    else:
        oops("VERSION attribute missing")
    header = read_header(h5)
    print("Header:", header)
    if not "data" in h5:
        oops("data attribute missing")
    if h5["data"].ndim != 3:
        oops("Expected data.ndim to be 3 but saw '{}'".format(h5["data"].ndim))
    print("srcname: data shape:", h5["data"].shape)
    h5.close()

    return header


def cmd_tool(args=None):
    """ Command line tool srcname """
    parser = ArgumentParser(description="Patch the header source field in an HDF5 file.")
    parser.add_argument("filepath", default=None, type=str, help="Path of file to read")
    parser.add_argument("new_source_name", default=None, type=str, 
                        help="New header source name field value")
    
    if args is None:
        args = parser.parse_args()
    else:
        args = parser.parse_args(args)

    if not os.path.isfile(args.filepath):
        oops("Not a file: {}".format(args.filepath))
    if not h5py.is_hdf5(args.filepath):
        oops("Not an HDF5 file: {}".format(args.filepath))

    header = examine(args.filepath)
    print("srcname: No errors detected in {}".format(args.filepath))
    print("\nThe current source_name field is [{}]".format(header["source_name"]))
    input("Are you sure you want to replace it with [{}]?  Press Enter to continue.  Ctrl-C to cancel: "
          .format(args.new_source_name))
    h5 = h5py.File(args.filepath, mode="r+")
    h5["data"].attrs["source_name"] = args.new_source_name
    h5.close()
    print("All done, best wishes!")


if __name__ == "__main__":
    cmd_tool()



import os
import sys
import time
import h5py
import hdf5plugin
from blimpy import utils


def write_to_hdf5(wf, filename_out, f_scrunch=None, *args, **kwargs):
    """ Copy the header and the selected data matrix subset from the input file to the output HDF5 file.
        Check the heavy flag to decide how to write the file - light or heavy.

    Args:
        wf : Waterfall object
        filename_out (str): Name of output file
        f_scrunch (int or None): Average (scrunch) N channels together
    """

    #For timing how long it takes to write a file.
    t0 = time.time()

    #Update header
    wf._update_header()

    if wf.container.isheavy():
        __write_to_hdf5_heavy(wf, filename_out, f_scrunch=f_scrunch, *args, **kwargs)
    else:
        __write_to_hdf5_light(wf, filename_out, f_scrunch=f_scrunch, *args, **kwargs)

    t1 = time.time()
    wf.logger.info('Conversion time: %2.2fsec' % (t1- t0))


def __write_to_hdf5_heavy(wf, filename_out, f_scrunch=None, *args, **kwargs):
    """ Copy the header and the selected data matrix subset from the input file to the output HDF5 file, blob by blob.

    Args:
        wf : Waterfall object
        filename_out (str): Name of output file
        f_scrunch (int or None): Average (scrunch) N channels together
    """

    # Note that a chunk is not a blob!!
    # chunk_dim = wf._get_chunk_dimensions() <-- seems intended for raw to fil
    # And, chunk dimensions should not exceed the Waterfall selection shape dimensions.
    chunk_list = list(wf._get_chunk_dimensions())
    for ix in range(0, len(chunk_list)):
        if chunk_list[ix] > wf.selection_shape[ix]:
            chunk_list[ix] = wf.selection_shape[ix]
    chunk_dim = tuple(chunk_list)
    blob_dim  = wf._get_blob_dimensions(chunk_dim)
    n_blobs   = wf.container.calc_n_blobs(blob_dim)
    wf.logger.info("__write_to_hdf5_heavy: For {}, chunk_dim={}, blob_dim={}, n_blobs={}"
                   .format(filename_out, chunk_dim, blob_dim, n_blobs))

    # ===============================
    # Attempt to write the HDF5 file.
    # ===============================
   
    try:
        os.remove(filename_out)  # Try to pre-remove output .h5 file.
    except:
        pass
    
    try:
        with h5py.File(filename_out, 'w') as h5:
    
            h5.attrs['CLASS'] = 'FILTERBANK'
            h5.attrs['VERSION'] = '1.0'
    
            bs_compression = hdf5plugin.Bitshuffle(nelems=0, lz4=True)['compression']
            bs_compression_opts = hdf5plugin.Bitshuffle(nelems=0, lz4=True)['compression_opts']
    
            dout_shape     = list(wf.selection_shape)    # Make sure not a tuple
            dout_chunk_dim = list(chunk_dim)
    
            if f_scrunch is not None:
                dout_shape[-1] //= f_scrunch
                dout_chunk_dim[-1] //= f_scrunch
                wf.header['foff'] *= f_scrunch
    
            dset = h5.create_dataset('data',
                                     shape=tuple(dout_shape),
                                     chunks=tuple(dout_chunk_dim),
                                     compression=bs_compression,
                                     compression_opts=bs_compression_opts,
                                     dtype=wf.data.dtype)
    
            dset_mask = h5.create_dataset('mask',
                                          shape=tuple(dout_shape),
                                          chunks=tuple(dout_chunk_dim),
                                          compression=bs_compression,
                                          compression_opts=bs_compression_opts,
                                          dtype='uint8')
    
            dset.dims[2].label = b"frequency"
            dset.dims[1].label = b"feed_id"
            dset.dims[0].label = b"time"
    
            dset_mask.dims[2].label = b"frequency"
            dset_mask.dims[1].label = b"feed_id"
            dset_mask.dims[0].label = b"time"
    
            # Copy over header information (Filterbank metadata) as HDF5 attributes
            for key, value in wf.header.items():
                dset.attrs[key] = value
    
            # Compare blob_dim along frequency axis
            # to the same of the data matrix selection.
            if blob_dim[wf.freq_axis] < wf.selection_shape[wf.freq_axis]:
                
                # Blob frequency axis is SHORTER THAN that of the data matrix selection.
    
                for ii in range(0, n_blobs):
                    wf.logger.info('__write_to_hdf5_heavy: Processing blob %i of %i' % (ii + 1, n_blobs))
    
                    # Read next data matrix blob from the input file.
                    bob = wf.container.read_blob(blob_dim, n_blob=ii)
    
                    # Using channel index values for c_start and c_stop.
                    c_start = wf.container.chan_start_idx + ii * blob_dim[wf.freq_axis]
                    t_start = wf.container.t_start + (c_start / wf.selection_shape[wf.freq_axis]) * blob_dim[wf.time_axis]
                    t_stop = t_start + blob_dim[wf.time_axis]  
                    c_start = (c_start) % wf.selection_shape[wf.freq_axis]
                    c_stop = c_start + blob_dim[wf.freq_axis]

                    # Frequency scrunching?    
                    if f_scrunch is not None:
                        c_start //= f_scrunch
                        c_stop  //= f_scrunch
                        bob = utils.rebin(bob, n_z=f_scrunch)
    
                    # Store blob in the output data.    
                    dset[t_start:t_stop, 0, c_start:c_stop] = bob[:]
                    wf.logger.debug('__write_to_hdf5_heavy t_start={}, t_stop={}, c_start={}, c_stop={}'
                                    .format(t_start, t_stop, c_start, c_stop))
    
            else:
                
                # Blob frequency axis is LONGER THAN or EQUAL TO that of the data matrix selection.
    
                for ii in range(0, n_blobs):
                    wf.logger.info('__write_to_hdf5_heavy: Processing blob %i of %i' % (ii + 1, n_blobs))
    
                    # Read next data matrix blob from the input file.
                    bob = wf.container.read_blob(blob_dim, n_blob=ii)
                    
                    # Compute start time.
                    t_start = wf.container.t_start + ii * blob_dim[wf.time_axis]
    
                    # Compute stop time.
                    if (ii + 1) * blob_dim[wf.time_axis] > wf.n_ints_in_file:
                        # Last blob: smaller than the others in the time dimension.
                        t_stop = wf.n_ints_in_file
                    else:
                        # All but the last blob: Full-sized blob in the time dimension.
                        t_stop = (ii + 1) * blob_dim[wf.time_axis]
    
                    # Frequency scrunching?    
                    if f_scrunch is not None:
                        bob = utils.rebin(bob, n_z=f_scrunch)

                    # Store blob in the output data.
                    # Note that the entire selected frequency dimension is used
                    # instead of the blob frequency dimension.
                    dset[t_start:t_stop] = bob[:]
                    wf.logger.debug('__write_to_hdf5_heavy t_start={}, t_stop={}'
                                    .format(t_start, t_stop))

        # =============================================
        # Success!  The .h5 file is written and closed.
        # =============================================

    except Exception as ex1: # Something went wrong!
        wf.logger.error("__write_to_hdf5_heavy: Writing the output HDF5 file {} failed!"
                       .format(filename_out))
        print(repr(ex1))
        try:
            if os.path.exists(filename_out):
                os.remove(filename_out) # scrap a potentially corrupted HDF5 file.
                # Removal succeeded.  Exit to the O/S with a nonzero exit code.
                wf.logger.info("__write_to_hdf5_heavy: Removal of partial HDF5 file {} succeeded."
                               .format(filename_out))
            sys.exit(86)
        except Exception as ex2:
            wf.logger.error("__write_to_hdf5_heavy: Removal of partial HDF5 file {} failed!"
                           .format(filename_out))
            print(repr(ex2))
            sys.exit(86)


def __write_to_hdf5_light(wf, filename_out, f_scrunch=None, *args, **kwargs):
    """ Copy the header and the selected data matrix subset from the input file to the output HDF5 file in one go.

    Args:
        wf : Waterfall object
        filename_out (str): Name of output file
        f_scrunch (int or None): Average (scrunch) N channels together
    """

    wf.logger.info("__write_to_hdf5_light: Writing the spectra matrix for {} without blobbing."
                   .format(filename_out))
    try:
        os.remove(filename_out)  # Try to pre-remove output .h5 file.
    except:
        pass # Ignore errors.
    
    with h5py.File(filename_out, 'w') as h5:

        h5.attrs['CLASS']   = 'FILTERBANK'
        h5.attrs['VERSION'] = '1.0'

        bs_compression = hdf5plugin.Bitshuffle(nelems=0, lz4=True)['compression']
        bs_compression_opts = hdf5plugin.Bitshuffle(nelems=0, lz4=True)['compression_opts']

        # Frequency scrunching?
        if f_scrunch is None:
            data_out = wf.data # No, just copy as-is from SIGPROC Filterbank (.fil) selection
        else:
            wf.logger.info('Frequency scrunching by %i' % f_scrunch)
            data_out = utils.rebin(wf.data, n_z=f_scrunch)
            wf.header['foff'] *= f_scrunch

        dset = h5.create_dataset('data',
                                 data=data_out,
                                 compression=bs_compression,
                                 compression_opts=bs_compression_opts)

        dset_mask = h5.create_dataset('mask',
                                      shape=data_out.shape,
                                      compression=bs_compression,
                                      compression_opts=bs_compression_opts,
                                      dtype='uint8')

        dset.dims[2].label = b"frequency"
        dset.dims[1].label = b"feed_id"
        dset.dims[0].label = b"time"

        dset_mask.dims[2].label = b"frequency"
        dset_mask.dims[1].label = b"feed_id"
        dset_mask.dims[0].label = b"time"

        # Copy over header information (Filterbank metadata) as HDF5 attributes
        for key, value in wf.header.items():
            dset.attrs[key] = value



"""
Simple script for quickly peeking at the data matrix.
"""

import sys
from argparse import ArgumentParser

from blimpy import Waterfall


def oops(msg):
    print("\n*** OOPS, {} !!!".format(msg))
    sys.exit(86)


def cmd_tool(args=None):
    """ Command line utility for peeking at the data matrix.
    """

    parser = ArgumentParser(description="Command line utility for peeking at a .fil or .h5 file")
    parser.add_argument("filepath_in", type=str, help="Path of input .fil or .h5 file")
    parser.add_argument("-i", "--start_tint", dest="start_tint", type=int, default=0,
                        help="Starting integration index relative to 0.  Default: 0")
    parser.add_argument("-Z", "--if", dest="the_IF", type=int, default=0,
                        help="Starting IF index relative to 0.  Default: 0")
    parser.add_argument("-c", "--start_fchan", dest="start_fchan", type=int, default=0,
                        help="Starting fine channel index relative to 0.  Default: 0")


    if args is None:
        args = parser.parse_args()
    else:
        args = parser.parse_args(args)

    wf = Waterfall(args.filepath_in)

    if args.start_tint < 0 or args.start_tint > (wf.n_ints_in_file - 2):
        oops("--start_tint is not a valid time integration index")
    if args.the_IF < 0:
        oops("--if is not a valid IF index")
    if args.start_fchan < 0 or args.start_fchan > (wf.header["nchans"] - 3):
        oops("--start_fchan is not a valid fine channel index")

    print("Fine channel frequency columns     go this way ------->")
    print("Integration_{}:                     {}   {}   {}"
          .format(args.start_tint,
                  wf.data[args.start_tint, args.the_IF, args.start_fchan],
                  wf.data[args.start_tint, args.the_IF, args.start_fchan + 1],
                  wf.data[args.start_tint, args.the_IF, args.start_fchan + 2]))
    print("Integration_{}:                     {}   {}   {}"
          .format(args.start_tint + 1,
                  wf.data[args.start_tint + 1, args.the_IF, args.start_fchan],
                  wf.data[args.start_tint + 1, args.the_IF, args.start_fchan + 1],
                  wf.data[args.start_tint + 1, args.the_IF, args.start_fchan + 2]))


if __name__ == "__main__":
    cmd_tool()



import pytest
import blimpy as bl
from tests.data import voyager_fil

def test_reset_works():
    """
    The test is trivial, but coverage must be
    increased at all costs.
    """
    a, b = bl.match_fils.reset_outs()
    assert a is None
    assert b is None

def test_batcher():
    """
    This test may be obsolete once/if test_cmd
        can be patched up.
    """
    bl.match_fils.make_batch_script()

def test_find_header_size():
    assert bl.match_fils.find_header_size(voyager_fil) > 0

def test_cmdline():
    args = [voyager_fil, voyager_fil]
    with pytest.raises(OSError):
        bl.match_fils.cmd_tool(args)




import os
from blimpy import dice
import pytest

from tests.data import voyager_h5
from tests.data import voyager_fil
TEST_DATA_DIR = os.path.dirname(voyager_h5)
HERE = os.getcwd()

# This section makes sure that the tool exits correctly
# when certain issues are encountered.

def test_no_args():
    """
    Make sure the tool closes if no arguments are passed in
    """
    with pytest.raises(SystemExit):
        dice.cmd_tool()

def test_no_input_file():
    with pytest.raises(SystemExit):
        args = ['-b', '8419.24', '-e', '8419.35', '-x', 'h5', '-o', 'test_dice.h5']
        dice.cmd_tool(args)

def test_missing_format_type():
    with pytest.raises(SystemExit):
        args = ['-f', voyager_h5, '-b', '8419.24', '-e', '8419.35', '-x']
        dice.cmd_tool(args)

# This section makes sure that the tool can handle various
# file formats without exception.

def test_h5():
    os.chdir(TEST_DATA_DIR)
    args = ['-f', voyager_h5, '-b', '8419.24', '-e', '8419.35', '-x', 'h5', '-o', 'test_dice.h5']
    dice.cmd_tool(args)
    os.chdir(HERE)

def test_h5_no_out_file():
    os.chdir(TEST_DATA_DIR)
    args = ['-f', voyager_h5, '-b', '8419.24', '-e', '8419.35', '-x', 'h5']
    dice.cmd_tool(args)
    os.chdir(HERE)

def test_fil():
    os.chdir(TEST_DATA_DIR)
    args = ['-f', voyager_fil, '-b', '8419.24', '-e', '8419.35', '-x', 'fil', '-o', 'test_dice.fil']
    dice.cmd_tool(args)
    os.chdir(HERE)

def test_fil_no_out_file():
    os.chdir(TEST_DATA_DIR)
    args = ['-f', voyager_fil, '-b', '8419.24', '-e', '8419.35', '-x', 'fil']
    dice.cmd_tool(args)
    os.chdir(HERE)

